package com.ruoyi.ss.article.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.article.domain.SmArticle;
import com.ruoyi.ss.article.domain.SmArticleDto;
import com.ruoyi.ss.article.mapper.SmArticleMapper;
import com.ruoyi.ss.article.service.ISmArticleService;
import com.ruoyi.system.mapper.SysDictDataMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 文章Service业务层处理
 *
 * @author qiuzhenzhao
 * @date 2023-12-06
 */
@Service
public class SmArticleServiceImpl implements ISmArticleService
{
    @Resource
    private SmArticleMapper asArticleMapper;

    @Resource
    private SysDictDataMapper dictDataMapper;

    /**
     * 查询文章
     *
     * @param articleId 文章主键
     * @return 文章
     */
    @Override
    public SmArticle selectSmArticleByArticleId(Long articleId)
    {
        SmArticle smArticle = asArticleMapper.selectSmArticleByArticleId(articleId);
        if(ObjectUtils.isNotEmpty(smArticle)){
            smArticle.setFormatCreateTime(DateUtils.getYYYY_MM_DD(smArticle.getCreateTime()));
        }
        return smArticle;
    }

    /**
     * 查询文章列表
     *
     * @param smArticle 文章
     * @return 文章
     */
    @Override
    public List<SmArticle> selectSmArticleList(SmArticleDto smArticle)
    {
        List<SmArticle> smArticles = asArticleMapper.selectSmArticleList(smArticle);
        for (SmArticle smArticle1 : smArticles) {
            smArticle.setFormatCreateTime(DateUtils.getYYYY_MM_DD(smArticle1.getCreateTime()));
        }
        return smArticles;
    }

    /**
     * 新增文章
     *
     * @param smArticle 文章
     * @return 结果
     */
    @Override
    public int insertSmArticle(SmArticle smArticle)
    {
        smArticle.setCreateTime(DateUtils.getNowDate());
        return asArticleMapper.insertSmArticle(smArticle);
    }

    /**
     * 修改文章
     *
     * @param smArticle 文章
     * @return 结果
     */
    @Override
    public int updateSmArticle(SmArticle smArticle)
    {
        smArticle.setUpdateTime(DateUtils.getNowDate());
        return asArticleMapper.updateSmArticle(smArticle);
    }

    /**
     * 批量删除文章
     *
     * @param articleIds 需要删除的文章主键
     * @return 结果
     */
    @Override
    public int deleteSmArticleByArticleIds(Long[] articleIds)
    {
        return asArticleMapper.deleteSmArticleByArticleIds(articleIds);
    }

    /**
     * 删除文章信息
     *
     * @param articleId 文章主键
     * @return 结果
     */
    @Override
    public int deleteSmArticleByArticleId(Long articleId)
    {
        return asArticleMapper.deleteSmArticleByArticleId(articleId);
    }

    /**
     * 根据文章id获取标签列表
     *
     * @param articleId 文章主键
     * @return 结果
     */
    @Override
    public String[] getTagList(Long articleId) {
        SmArticle smArticle = asArticleMapper.selectSmArticleByArticleId(articleId);
        String tag = smArticle.getTag();
        String[] split = tag.split(",");
        return split;
    }

//    /**
//     * 查询动态分类及数量
//     *
//     * @param
//     * @return 结果
//     */
//    @Override
//    public List<DynamicClassify> selectDynamicClassifyList() {
//        List<DynamicClassify> classifies = new ArrayList<>();
//        SysDictData dictData = new SysDictData();
//        dictData.setDictType("dynamic_classify");
//        List<SysDictData> sysDictData = dictDataMapper.selectDictDataList(dictData);
//        for (SysDictData one :sysDictData) {
//            DynamicClassify dynamicClassify = new DynamicClassify();
//            String dictValue = one.getDictValue();
//            dynamicClassify.setName(one.getDictLabel());
//            dynamicClassify.setValue(dictValue);
//            int classify = asArticleMapper.countDynamicClassify(dictValue);
//            dynamicClassify.setNum(classify);
//            classifies.add(dynamicClassify);
//        }
//        return classifies;
//    }
}
