package com.ruoyi.ss.businessRecord.service;

import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.businessRecord.domain.SmBusinessRecord;
import com.ruoyi.ss.businessRecord.domain.SmBusinessRecordDto;
import com.ruoyi.ss.businessRecord.domain.SmBusinessRecordVo;
import com.ruoyi.ss.businessRecord.mapper.SmBusinessRecordMapper;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.dashboard.BriefVo;
import com.ruoyi.ss.device.domain.SmDeviceDto;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.model.domain.SmModelDto;
import com.ruoyi.ss.model.service.ISmModelService;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillDto;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.service.ISmTransactionBillService;
import com.ruoyi.ss.user.domain.SmUserDto;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 经营记录Service业务层处理
 *
 * @author ruoyi
 * @date 2024-03-19
 */
@Service
public class SmBusinessRecordServiceImpl implements ISmBusinessRecordService
{
    @Autowired
    private SmBusinessRecordMapper smBusinessRecordMapper;

    @Autowired
    private ISmDeviceService deviceService;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ISmModelService modelService;

    @Autowired
    private ISmTransactionBillService transactionBillService;

    /**
     * 查询经营记录
     *
     * @param id 经营记录主键
     * @return 经营记录
     */
    @Override
    public SmBusinessRecordVo selectSmBusinessRecordById(Long id)
    {
        return smBusinessRecordMapper.selectSmBusinessRecordById(id);
    }

    /**
     * 查询经营记录列表
     *
     * @param smBusinessRecord 经营记录
     * @return 经营记录
     */
    @Override
    public List<SmBusinessRecordVo> selectSmBusinessRecordList(SmBusinessRecordDto smBusinessRecord)
    {
        return smBusinessRecordMapper.selectSmBusinessRecordList(smBusinessRecord);
    }

    /**
     * 新增经营记录
     *
     * @param smBusinessRecord 经营记录
     * @return 结果
     */
    @Override
    public int insertSmBusinessRecord(SmBusinessRecord smBusinessRecord)
    {
        smBusinessRecord.setCreateTime(DateUtils.getNowDate());
        return smBusinessRecordMapper.insertSmBusinessRecord(smBusinessRecord);
    }

    /**
     * 修改经营记录
     *
     * @param smBusinessRecord 经营记录
     * @return 结果
     */
    @Override
    public int updateSmBusinessRecord(SmBusinessRecord smBusinessRecord)
    {
        return smBusinessRecordMapper.updateSmBusinessRecord(smBusinessRecord);
    }

    /**
     * 批量删除经营记录
     *
     * @param ids 需要删除的经营记录主键
     * @return 结果
     */
    @Override
    public int deleteSmBusinessRecordByIds(Long[] ids)
    {
        return smBusinessRecordMapper.deleteSmBusinessRecordByIds(ids);
    }

    /**
     * 删除经营记录信息
     *
     * @param id 经营记录主键
     * @return 结果
     */
    @Override
    public int deleteSmBusinessRecordById(Long id)
    {
        return smBusinessRecordMapper.deleteSmBusinessRecordById(id);
    }

    @Override
    public SmBusinessRecordVo selectLatestByDate(Date date) {
        SmBusinessRecordDto dto = new SmBusinessRecordDto();
        dto.setCreateDate(date);
        return smBusinessRecordMapper.selectLatest(dto);
    }

    @Override
    public List<SmBusinessRecordVo> selectDailyLatest(Date startDate, Date endDate) {
        SmBusinessRecordDto dto = new SmBusinessRecordDto();
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        return smBusinessRecordMapper.selectDailyLatest(dto);
    }

    /**
     * 查询实时数据
     */
    @Override
    public BriefVo selectBrief() {
        BriefVo brief = new BriefVo();

        // 设备总数
        SmDeviceDto deviceDto = new SmDeviceDto();
        deviceDto.setDeleted(false);
        deviceDto.setIsBind(true);
        brief.setDeviceCount(deviceService.selectCount(deviceDto));

        // 在线设备总数
        SmDeviceDto onlineDto = new SmDeviceDto();
        onlineDto.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        onlineDto.setDeleted(false);
        onlineDto.setIsBind(true);
        brief.setOnlineCount(deviceService.selectCount(onlineDto));

        // 型号总数
        SmModelDto modelDto = new SmModelDto();
        modelDto.setDeleted(false);
        brief.setModelCount(modelService.selectCount(modelDto));

        // 房东总数
        SmUserDto userDto = new SmUserDto();
        userDto.setDelFlag(UserStatus.OK.getCode());
        userDto.setUserType(UserType.LANDLORD.getType());
        brief.setUserCount(userService.selectCount(userDto));

        // 租户总数
        SmUserDto tenantDto = new SmUserDto();
        tenantDto.setDelFlag(UserStatus.OK.getCode());
        tenantDto.setUserType(UserType.TENANT.getType());
        brief.setTenantCount(userService.selectCount(tenantDto));

        // 充值金额总数
        SmTransactionBillDto rechargeDto = new SmTransactionBillDto();
        rechargeDto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        List<BillCountVo> billCount = transactionBillService.selectCount(rechargeDto);
        if (!CollectionUtils.isEmpty(billCount)) {
            brief.setRechargeAmount(billCount.get(0).getRecharge());
        }

        return brief;
    }

    /**
     * 记录数据
     */
    @Override
    public void record() {
        BriefVo brief = this.selectBrief();

        SmBusinessRecord data = new SmBusinessRecord();
        data.setTotalDevice(brief.getDeviceCount());
        data.setTotalOnline(brief.getOnlineCount());
        data.setTotalModel(brief.getModelCount());
        data.setTotalUser(brief.getUserCount() + brief.getTenantCount());
        data.setTotalRecharge(brief.getRechargeAmount());
        this.insertSmBusinessRecord(data);
    }
}
