package com.ruoyi.ss.device.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.valid.EnumValid;
import com.ruoyi.ss.device.domain.enums.DeviceOutageWay;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Range;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 设备对象 sm_device
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Data
public class SmDevice extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 设备id */
    @ApiModelProperty("设备ID")
    @NotNull(message = "设备id不允许为空", groups = {ValidGroup.Update.class})
    private Long deviceId;

    /** 分组id */
    @Excel(name = "分组id")
    @ApiModelProperty("分组id")
    private Long groupId;

    /** 设备名称 */
    @Excel(name = "设备名称")
    @ApiModelProperty("设备名称")
    @NotNull(message = "设备名称不允许为空", groups = {ValidGroup.Create.class, ValidGroup.Update.class})
    private String deviceName;

    /** 型号id */
    @Excel(name = "型号id")
    @ApiModelProperty("型号id")
    @NotNull(message = "型号不允许为空", groups = {ValidGroup.Create.class, ValidGroup.Update.class})
    private Long modelId;


    /** 设备Mac号 */
    @Excel(name = "设备Mac号")
    @ApiModelProperty("设备Mac号")
    private String mac;

    /** 激活时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("激活时间")
    @Excel(name = "激活时间", width = 30, dateFormat = "yyyy-MM-dd")
    @Past(message = "激活时间必须是过去的某个时间", groups = {ValidGroup.Create.class, ValidGroup.Update.class, ValidGroup.FrontUpdate.class})
    private Date activationTime;

    /** 剩余电量（度） */
    @Excel(name = "剩余电量", readConverterExp = "度=")
    @ApiModelProperty("剩余电量")
    private BigDecimal surplusElectriQuantity;

    @Excel(name = "总用电量")
    @ApiModelProperty("总用电量（KWH）")
    private BigDecimal totalElectriQuantity;

    /** 在线状态 */
    @Excel(name = "在线状态")
    @ApiModelProperty("在线状态")
    private String onlineStatus;

    /** 状态 */
    @Excel(name = "状态")
    @ApiModelProperty("状态")
    private String status;

    /** 实时功率（千瓦） */
    @Excel(name = "实时功率", readConverterExp = "千=瓦")
    @ApiModelProperty("实时功率")
    private BigDecimal realTimePower;

    /** 电流（A） */
    @Excel(name = "电流", readConverterExp = "A=")
    @ApiModelProperty("电流（A）")
    @Min(value = 0, message = "电流不允许小于0", groups = {ValidGroup.Create.class, ValidGroup.Update.class, ValidGroup.FrontUpdate.class})
    private BigDecimal electricity;

    /** 电压（V） */
    @Excel(name = "电压", readConverterExp = "V=")
    @ApiModelProperty("电压（V）")
    @Min(value = 0, message = "电压不允许小于0", groups = {ValidGroup.Create.class, ValidGroup.Update.class, ValidGroup.FrontUpdate.class})
    private BigDecimal voltage;

    /** 所属房东ID */
    @Excel(name = "所属房东ID")
    @ApiModelProperty("所属房东ID")
    @NotNull(message = "房东不允许为空", groups = {ValidGroup.Create.class, ValidGroup.Update.class})
    private Long userId;

    /** 用户昵称 */
    @Excel(name = "用户昵称")
    @ApiModelProperty("用户昵称")
    private String nickName;

    @Excel(name = "电费单价（元/度）")
    @ApiModelProperty("电费单价（元/度）")
    @Min(value = 0, message = "电费单价不允许小于0", groups = {ValidGroup.Create.class, ValidGroup.Update.class, ValidGroup.FrontUpdate.class})
    private BigDecimal price;

    @Excel(name = "欠费断电")
    @ApiModelProperty("断电方式：0欠费不断电，1欠费立即断电")
    @EnumValid(
            clazz = DeviceOutageWay.class,
            message = "非法的欠费断电方式",
            method = "getValue",
            groups = {ValidGroup.Create.class, ValidGroup.Update.class, ValidGroup.FrontUpdate.class}
    )
    private String outageWay;

    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "下次房租到期提醒日期")
    @ApiModelProperty("下次房租到期提醒日期")
    private Date expireDate;

    @Excel(name = "wifi名称")
    @ApiModelProperty("wifi名称")
    private String wifi;

    @Excel(name = "是否开启房租到期提醒")
    @ApiModelProperty("是否开启房租到期提醒")
    private Boolean enableExpireNotice;

    @Excel(name = "房租到期提醒方式")
    @ApiModelProperty("房租到期提醒方式")
    private String noticeWay;

    @ApiModelProperty("是否由用户承担服务费")
    private Boolean tenantBearServiceFee;

    @ApiModelProperty("上次拉取设备信息的时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastPullTime;

    @ApiModelProperty("房租到期提醒日（每月）")
    @Range(min = 1, max = 31, message = "房租到期提醒日应该在1到31日（含）之间")
    private Integer expireDay;

    @ApiModelProperty("电量初始读数")
    private BigDecimal initReading;

    @ApiModelProperty("开关状态,0断电，1通电")
    private String powerStatus;
}
