package com.ruoyi.ss.device.mapper;

import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceDto;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 设备Mapper接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface SmDeviceMapper
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public SmDeviceVo selectSmDeviceByDeviceId(Long deviceId);

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备集合
     */
    public List<SmDeviceVo> selectSmDeviceList(SmDeviceDto smDevice);

    /**
     * 新增设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int insertSmDevice(SmDevice smDevice);

    /**
     * 修改设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int updateSmDevice(SmDevice smDevice);

    /**
     * 删除设备
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    public int deleteSmDeviceByDeviceId(Long deviceId);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 条件查询数量
     * @param dto 条件
     */
    int selectCount(SmDeviceDto dto);

    /**
     * 增加设备电量
     * @param deviceId 设备
     * @param num 增加的量
     */
    int addElectricity(@Param("deviceId") Long deviceId, @Param("num") BigDecimal num);

    /**
     * 逻辑删除
     */
    int logicDel(@Param("deviceIds") List<Long> deviceIds);

    /**
     * 查询简单数据，不联表
     * @param dto
     * @return
     */
    List<SmDeviceVo> selectSimpleList(SmDeviceDto dto);

    /**
     * 设备绑定房东
     * @param deviceId 设备id
     * @param userId 房东id
     */
    int bindLandlord(@Param("deviceId") Long deviceId, @Param("userId") Long userId);

    /**
     * 解绑房东
     * @param deviceId 设备id
     * @param userId 房东id
     */
    int unbindLandlord(@Param("deviceId") Long deviceId, @Param("userId") Long userId);

    /**
     * 根据MAC查询设备
     * @param mac
     * @return
     */
    SmDeviceVo selectSimpleSmDeviceByMac(String mac);
}
