package com.ruoyi.ss.device.service;

import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroupVo;
import com.ruoyi.ss.deviceGroup.service.ISmDeviceGroupService;
import com.ruoyi.ss.deviceTenant.domain.SmDeviceTenant;
import com.ruoyi.ss.deviceTenant.service.ISmDeviceTenantService;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/4/13
 */
@Service
public class DeviceAssemblerImpl implements DeviceAssembler {

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ISmDeviceTenantService deviceTenantService;

    @Autowired
    private ISmDeviceGroupService smDeviceGroupService;

    @Override
    public void assembleTenant(List<SmDeviceVo> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        Map<Long, List<SmDeviceTenant>> tenantGroup = deviceTenantService.selectGroupByDeviceIds(list.stream().map(SmDeviceVo::getDeviceId).collect(Collectors.toList()));

        for (SmDeviceVo device : list) {
            List<SmDeviceTenant> tenants = tenantGroup.get(device.getDeviceId());
            if (CollectionUtils.isEmpty(tenants)) {
                continue;
            }
            device.setTenantIds(tenants.stream().map(SmDeviceTenant::getTenantId).collect(Collectors.toList()));
        }

    }

    @Override
    public void setGroup(SmDevice device) {
        // 房东的默认分组
        if (device.getUserId() != null && device.getGroupId() == null) {
            SmDeviceGroupVo defaultGroup = smDeviceGroupService.selectDefaultGroup(device.getUserId());
            if (defaultGroup == null) {
                defaultGroup = smDeviceGroupService.insertDefaultGroup(device.getUserId());
                ServiceUtil.assertion(defaultGroup == null, "该房东没有默认分组");
            }
            device.setGroupId(defaultGroup.getGroupId());
        }
    }
}
