package com.ruoyi.ss.device.service;

import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceDto;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * 设备Service接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface ISmDeviceService
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public SmDeviceVo selectSmDeviceByDeviceId(Long deviceId);

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备集合
     */
    public List<SmDeviceVo> selectSmDeviceList(SmDeviceDto smDevice);

    /**
     * 新增设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int insertSmDevice(SmDevice smDevice);

    /**
     * 修改设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int updateSmDevice(SmDevice smDevice);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键集合
     * @return 结果
     */
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 条件查询数量
     * @param dto 查询条件
     * @return 结果
     */
    int selectCount(SmDeviceDto dto);

    /**
     * 查询指定分组下的设备列表
     * @param groupId
     */
    List<SmDeviceVo> selectSmDeviceByGroupId(Long groupId);

    /**
     * 增加电量
     * @param deviceId 设备id
     * @param num 电量
     */
    boolean addElectricity(Long deviceId, BigDecimal num);

    /**
     * 租户绑定设备
     * @param deviceId 设备id
     */
    boolean tenantBind(Long tenantId, Long deviceId);

    /**
     * 租户解绑设备
     * @param userId 租户id
     * @param deviceId 设备id
     */
    boolean tenantUnbind(Long userId, Long deviceId);

    /**
     * 获取用户的默认设备
     * @param userId 用户id
     */
    public SmDeviceVo selectDefaultDevice(Long userId);

    /**
     * 设置用户默认设备
     * @param tenantId 租户id
     * @param deviceId 设备id
     */
    boolean setDefaultDevice(Long tenantId, Long deviceId);

    /**
     * 逻辑删除
     */
    boolean logicDel(List<Long> deviceIds);

    /**
     * 通过型号id列表，查询设备列表，并分组
     * @param modelIds 型号id列表
     */
    Map<Long, List<SmDeviceVo>> selectGroupByModelIds(List<Long> modelIds);

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    boolean changePowerStatus(Long deviceId, DevicePowerStatus status);

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    void pullDeviceInfo(List<Long> deviceIds);

    /**
     * 通过设备id绑定设备
     * 设备第一次绑定为房东
     * 其他时候绑定为租户
     * @param userId 用户id
     * @param deviceId 设备id
     */
    boolean bind(Long userId, Long deviceId);

    /**
     * 通过mac绑定设备
     * 设备第一次绑定为房东
     * 其他时候绑定为租户
     * @param userId 用户id
     * @param mac 设备mac
     */
    boolean bind(Long userId, String mac);

    /**
     * 房东解绑设备
     * @param userId 用户id
     * @param deviceId 设备id
     */
    boolean landlordUnbind(Long userId, Long deviceId);

    /**
     * 通过mac查询设备
     * @param mac mac号
     * @return 设备信息
     */
    SmDeviceVo selectByMac(String mac);

    /**
     * 归零电表
     * @param deviceId 设备id
     */
    boolean reset(Long deviceId);

    /**
     * 设备是否已经被绑定
     * @param mac
     * @return
     */
    boolean isBind(String mac);
}
