package com.ruoyi.ss.transactionBill.mapper;

import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBill;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillDto;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillVo;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 充值记录Mapper接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface SmTransactionBillMapper
{
    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    public SmTransactionBill selectSmTransactionBillByBillId(Long billId);

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBill 充值记录
     * @return 充值记录集合
     */
    public List<SmTransactionBillVo> selectSmTransactionBillList(SmTransactionBillDto smTransactionBill);

    /**
     * 新增充值记录
     *
     * @param smTransactionBill 充值记录
     * @return 结果
     */
    public int insertSmTransactionBill(SmTransactionBill smTransactionBill);

    /**
     * 修改充值记录
     *
     * @param smTransactionBill 充值记录
     * @return 结果
     */
    public int updateSmTransactionBill(SmTransactionBill smTransactionBill);

    /**
     * 删除充值记录
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    public int deleteSmTransactionBillByBillId(Long billId);

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteSmTransactionBillByBillIds(Long[] billIds);

    /**
     * 获取账单统计数据
     * @param dto 查询条件
     */
    List<BillCountVo> selectCount(SmTransactionBillDto dto);

    /**
     * 修改支付方式
     * @param billId 订单id
     * @param payType 支付方式
     */
    int updatePayType(@Param("billId") Long billId, @Param("payType") Long payType, @Param("accountNo") String accountNo);

    /**
     * 充值成功
     * @param billId 订单id
     */
    int rechargeSuccess(@Param("billId") Long billId, @Param("payTime") Date payTime);

    /**
     * 通过订单编号查询订单
     * @param billNo 订单编号
     * @return
     */
    SmTransactionBill selectSmTransactionBillByBillNo(String billNo);

    /**
     * 订单取消
     * @param billNo 订单编号
     * @param status 订单取消后的状态
     * @return 影响的行数
     */
    int cancelRecharge(@Param("billNo") String billNo, @Param("status") String status);

    /**
     * 审核通过
     * @param billId id
     * @param remark 审核意见
     */
    int doPassWithdraw(@Param("billId") Long billId, @Param("remark") String remark);

    /**
     * 拒绝提现
     * @param billId id
     * @param remark 审核意见
     */
    int doRejectWithdraw(@Param("billId") Long billId, @Param("remark") String remark);

    /**
     * 发起提现打款（打款中...）
     * @param billId 提现订单id
     */
    int doPayWithdraw(Long billId);

    /**
     * 提现成功
     * @param billId id
     */
    int withdrawSuccess(Long billId);

    /**
     * 提现失败
     * @param billId id
     */
    int withdrawFailed(Long billId);

    /**
     * 更新状态为支付中
     * @param billId
     * @return
     */
    int paying(Long billId);

    int updateDeviceRechargeStatus(@Param("billId") Long billId, @Param("status") String status);
}
