package com.ruoyi.ss.transactionBill.service;

import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBill;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillBO;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillDto;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillVo;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillPayType;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;

import java.util.Date;
import java.util.List;

/**
 * 充值记录Service接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface ISmTransactionBillService
{
    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    public SmTransactionBill selectSmTransactionBillByBillId(Long billId);

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBillDto 充值记录查询条件
     * @return 充值记录集合
     */
    public List<SmTransactionBillVo> selectSmTransactionBillList(SmTransactionBillDto smTransactionBillDto);

    /**
     * 新增充值记录
     *
     * @param smTransactionBill 充值记录
     * @return 结果
     */
    public int insertSmTransactionBill(SmTransactionBill smTransactionBill);

    /**
     * 修改充值记录
     *
     * @param smTransactionBill 充值记录
     * @return 结果
     */
    public int updateSmTransactionBill(SmTransactionBill smTransactionBill);

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的充值记录主键集合
     * @return 结果
     */
    public int deleteSmTransactionBillByBillIds(Long[] billIds);

    /**
     * 删除充值记录信息
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    public int deleteSmTransactionBillByBillId(Long billId);

    /**
     * 获取账单统计
     * @param dto
     * @return
     */
    List<BillCountVo> selectCount(SmTransactionBillDto dto);

    /**
     * 创建充值订单
     * @param body 订单信息
     * @return 新增的数据订单编号
     */
    String addRecharge(SmTransactionBill body);

    /**
     * 更新支付方式
     * @param billId 订单id
     * @param transactionBillPayType 支付方式
     */
    int updatePayType(Long billId, TransactionBillPayType transactionBillPayType, String accountNo);

    /**
     * 获取支付结果
     * @param billNo
     * @return
     */
    boolean getPayResult(String billNo);

    /**
     * 充值成功
     * @param billNo 订单编号
     */
    void rechargeSuccess(String billNo, Date payTime);

    /**
     * 取消充值订单
     * @param billNo 订单编号
     * @param status 取消后的状态
     */
    int cancelRecharge(String billNo, TransactionBillStatus status);

    /**
     * 根据订单编号查询数据
     * @param billNo 订单编号
     */
    SmTransactionBill selectSmTransactionBillByBillNo(String billNo);

    /**
     * 获取未支付的充值订单
     * @return 订单列表
     */
    List<SmTransactionBillVo> selectUnpaidRecharge();

    /**
     * 提现
     */
    String addWithdraw(SmTransactionBillBO data);

    /**
     * 提现审核通过
     * @param billId id
     * @param remark 审核意见
     */
    boolean passWithdraw(Long billId, String remark);

    /**
     * 提现审核拒绝
     * @param billId id
     * @param remark 审核意见
     */
    boolean rejectWithdraw(Long billId, String remark);

    /**
     * 提现打款
     * @param billId 提现单id
     */
    boolean payWithdraw(Long billId);

    /**
     * 提现成功
     * @param billId id
     */
    void withdrawSuccess(Long billId);

    /**
     * 提现失败
     * @param billId id
     */
    void withdrawFailed(Long billId);

    /**
     * 更新微信批次明细单号
     * @param billId 提现单号
     * @param transferIds 微信批次明细列表
     */
    boolean updateWxTransferDetailIds(Long billId, List<String> transferIds);

    /**
     * 查询房东统计信息
     * @param dto 查询条件
     * @return 统计信息
     */
    List<BillCountVo> selectLandlordCount(SmTransactionBillDto dto);

    /**
     * 刷新订单支付结果
     * @param billIds
     */
    void refreshPayResult(List<Long> billIds);

    /**
     * 支付中
     * @param billId
     */
    void paying(Long billId);

    /**
     * 手动设备充值
     * @param billId
     * @param validate 是否需要校验
     */
    boolean rechargeDevice(Long billId, boolean validate);
}
