package com.ruoyi.ss.transactionBill.service;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.ss.account.domain.SmAccountVo;
import com.ruoyi.ss.account.service.ISmAccountService;
import com.ruoyi.ss.channel.domain.SmChannel;
import com.ruoyi.ss.channel.service.ISmChannelService;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.mapper.SmDeviceMapper;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBill;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillBO;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillDto;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillVo;
import com.ruoyi.ss.transactionBill.domain.enums.*;
import com.ruoyi.ss.transactionBill.mapper.SmTransactionBillMapper;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.mapper.SmUserMapper;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.ss.wxPay.service.IWxPayService;
import com.ruoyi.system.domain.enums.config.ConfigKey;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.task.bill.BillDelayedManager;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 充值记录Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service("transactionBillService")
@Slf4j
public class SmTransactionBillServiceImpl implements ISmTransactionBillService {
    @Autowired
    private SmTransactionBillMapper smTransactionBillMapper;

    @Autowired
    private ISmChannelService channelService;

    @Autowired
    private SmDeviceMapper deviceMapper;

    @Autowired
    private ISmDeviceService deviceService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private BillDelayedManager billDelayedManager;

    @Autowired
    private SmUserMapper smUserMapper;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ISmAccountService smAccountService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Value("${sm.withdraw.password.enable}")
    private Boolean smWithdrawPasswordEnable;   // 是否开启提现密码

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    @Override
    public SmTransactionBill selectSmTransactionBillByBillId(Long billId)
    {
        return smTransactionBillMapper.selectSmTransactionBillByBillId(billId);
    }

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBillDto 充值记录查询条件
     * @return 充值记录
     */
    @Override
    public List<SmTransactionBillVo> selectSmTransactionBillList(SmTransactionBillDto smTransactionBillDto)
    {
        return smTransactionBillMapper.selectSmTransactionBillList(smTransactionBillDto);
    }

    /**
     * 新增充值记录
     *
     * @param smTransactionBill 充值记录
     * @return 结果
     */
    @Override
    @Transactional
    public int insertSmTransactionBill(SmTransactionBill smTransactionBill)
    {
        smTransactionBill.setCreateTime(DateUtils.getNowDate());
        smTransactionBill.setBillNo(String.valueOf(SnowFlakeUtil.newId()));
        return smTransactionBillMapper.insertSmTransactionBill(smTransactionBill);
    }

    /**
     * 修改充值记录
     *
     * @param smTransactionBill 充值记录
     * @return 结果
     */
    @Override
    public int updateSmTransactionBill(SmTransactionBill smTransactionBill)
    {
        return smTransactionBillMapper.updateSmTransactionBill(smTransactionBill);
    }

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的充值记录主键
     * @return 结果
     */
    @Override
    public int deleteSmTransactionBillByBillIds(Long[] billIds)
    {
        return smTransactionBillMapper.deleteSmTransactionBillByBillIds(billIds);
    }

    /**
     * 删除充值记录信息
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    @Override
    public int deleteSmTransactionBillByBillId(Long billId)
    {
        return smTransactionBillMapper.deleteSmTransactionBillByBillId(billId);
    }

    /**
     * 查询账单统计
     * @param dto
     * @return
     */
    @Override
    public List<BillCountVo> selectCount(SmTransactionBillDto dto) {
        return smTransactionBillMapper.selectCount(dto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addRecharge(SmTransactionBill data) {
        SmDeviceVo device = deviceService.selectSmDeviceByDeviceId(data.getDeviceId());
        ServiceUtil.assertion(device == null || device.getDeleted(), "当前设备不存在");
        ServiceUtil.assertion(StringUtils.isEmpty(device.getMac()), "当前设备Mac为空，无法充值");
        ServiceUtil.assertion(!Objects.equals(device.getOnlineStatus(), DeviceOnlineStatus.ONLINE.getStatus()), "当前设备不在线，无法充值");

        // 判断是否租户或者房东
        deviceAssembler.assembleTenant(Collections.singletonList(device));
        List<Long> allowUsers = device.getTenantIds();
        if (allowUsers == null) {
            allowUsers = new ArrayList<>();
        }
        allowUsers.add(device.getUserId());
        ServiceUtil.assertion(!UserUtil.hasFrontUser(allowUsers), "您不是该设备的租户或房东");

        // 判断各种信息是否一致
        ServiceUtil.assertion(device.getPrice().compareTo(data.getUnitPrice()) != 0, "计费标准不一致，请重新下单");

        SmChannel channel = channelService.selectSmChannelByChannelId(data.getPayType());
        ServiceUtil.assertion(channel == null || !channel.getEnabled(), "不支持的支付方式");
        BigDecimal serviceRate = channel.getServiceRate().divide(new BigDecimal(100), 4, RoundingMode.HALF_UP);    // 渠道手续费率
        BigDecimal costRate = channel.getCostRate().divide(new BigDecimal(100), 4, RoundingMode.HALF_UP);      // 渠道成本率
        ServiceUtil.assertion(BigDecimal.ZERO.compareTo(serviceRate) > 0, "手续费费率不允许小于0");

        BigDecimal serviceCharge = serviceRate.multiply(data.getDeviceAmount());    // 服务费
        BigDecimal arrivalAmount = data.getDeviceAmount();  // 房东最终到账的金额
        BigDecimal payMoney = data.getDeviceAmount();  // 客户需要支付的金额
        if (device.getTenantBearServiceFee()) {
            // 若由客户承担服务费，则需要支付的金额 + 服务费
            payMoney = payMoney.add(serviceCharge);
        } else {
            // 由房东承担服务费，则房东到账金额 - 服务费
            arrivalAmount = arrivalAmount.subtract(serviceCharge);
        }
        ServiceUtil.assertion(serviceCharge.compareTo(data.getServiceCharge()) != 0, "手续费计算不一致，请重新下单");
        ServiceUtil.assertion(arrivalAmount.compareTo(data.getArrivalAmount()) != 0, "房东到账金额计算不一致，请重新下单");
        ServiceUtil.assertion(payMoney.compareTo(data.getMoney()) != 0, "待支付金额计算不一致，请重新下单");

        data.setLandlordId(device.getUserId()); // 到账人
        data.setType(TransactionBillType.RECHARGE.getType());
        data.setStatus(TransactionBillStatus.UNPAID.getStatus());
        // 过期时间
        long expireTime = TimeUnit.MILLISECONDS.convert(Constants.BILL_UNPAID_TIMEOUT, Constants.BILL_UNPAID_TIMEUNIT) + System.currentTimeMillis();
        data.setExpireTime(new Date(expireTime));
        // 渠道成本
        data.setChannelCost(costRate.multiply(payMoney));
        // 下单
        ServiceUtil.assertion(this.insertSmTransactionBill(data) != 1, "下单失败");

        // 加入延时队列，超时取消
        billDelayedManager.add(data.getBillNo(), expireTime);

        return data.getBillNo();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addWithdraw(SmTransactionBillBO data) {
        Long userId = data.getUserId();
        // 参数校验
        ServiceUtil.assertion(userId == null, "参数错误：用户id为空");

        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.ADD_WITHDRAW, userId.toString()), "请勿频繁操作提现");

            // 校验信息是否正确
            // 提现密码
            SmUserVo user = smUserMapper.selectSimpleById(userId);
            if (this.smWithdrawPasswordEnable) {
                ServiceUtil.assertion(StringUtils.isBlank(data.getPayPassword()), "提现密码不允许为空");
                ServiceUtil.assertion(StringUtils.isBlank(user.getPayPassword()), "用户未设置提现密码", 10001);
                ServiceUtil.assertion(!Objects.equals(user.getPayPassword(), SecurityUtils.encryptPassword(data.getPayPassword())), "提现密码错误");
            }

            // 判断该账号是不是本人绑定的提现账号
            TransactionBillPayType payType = TransactionBillPayType.parse(data.getPayType());
            String accountNo = data.getAccountNo();
            if (TransactionBillPayType.BANK.equals(payType)) {
                ServiceUtil.assertion(accountNo == null, "银行卡账户不允许为空");
                ServiceUtil.assertion(!smAccountService.validateUserHasAccount(userId, accountNo, payType.getAccountType()),
                        "该提现账号未绑定，请重新选择");
            } else if (TransactionBillPayType.WECHAT.equals(payType)){
                SmAccountVo wxAccount = smAccountService.selectUserWxAccount(userId);
                ServiceUtil.assertion(wxAccount == null, "用户不存在微信账户");
                accountNo = wxAccount.getAccountNo();
            } else if (TransactionBillPayType.ALI.equals(payType)) {
                SmAccountVo wxAccount = smAccountService.selectUseAliAccount(userId);
                ServiceUtil.assertion(wxAccount == null, "用户不存在支付宝账户");
                accountNo = wxAccount.getAccountNo();
            }

            // 计算各项金额是否准确
            BigDecimal serviceFeeRate = configService.getBigDecimal(ConfigKey.WITHDRAW_SERVICE_FEE_RATE);
            ServiceUtil.assertion(BigDecimal.ZERO.compareTo(serviceFeeRate) > 0, "手续费不允许小于0");
            BigDecimal serviceRecharge = data.getMoney().multiply(serviceFeeRate); // 服务费
            ServiceUtil.assertion(serviceRecharge.compareTo(data.getServiceCharge()) != 0, "手续费计算不一致，请重新申请");
            BigDecimal arrivalAmount = data.getMoney().subtract(serviceRecharge);   // 到账金额
            ServiceUtil.assertion(arrivalAmount.compareTo(data.getArrivalAmount()) != 0, "到账金额计算不一致，请重新申请");

            // 减少余额，并判断提现金额是否超额（减少的是交易金额）
            userService.subtractBalance(userId, data.getMoney());

            // 减少余额后的用户信息
            SmUserVo afterUser = smUserMapper.selectSimpleById(userId);

            // 创建申请订单
            SmTransactionBill bill = new SmTransactionBill();
            bill.setUserId(userId);
            bill.setType(TransactionBillType.WITHDRAW.getType());
            bill.setLandlordId(userId);
            bill.setMoney(data.getMoney());
            bill.setArrivalAmount(arrivalAmount);
            bill.setServiceCharge(serviceRecharge);
            bill.setStatus(TransactionBillStatus.WITHDRAW_APPROVING.getStatus());   // 审核中
            bill.setAfterBalance(afterUser.getBalance());
            bill.setAccountNo(accountNo);
            bill.setPayType(data.getPayType());
            ServiceUtil.assertion(this.insertSmTransactionBill(bill) != 1, "提现申请失败");

            return bill.getBillNo();
        } finally {
            redisLock.unlock(RedisLockKey.ADD_WITHDRAW, userId.toString());
        }

    }

    /**
     * 提现审核通过
     * @param billId id
     * @param remark 审核意见
     */
    @Override
    @Transactional
    public boolean passWithdraw(Long billId, String remark) {
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString()), "该提现申请正在审核，请刷新后重试");

            int updateCount = smTransactionBillMapper.doPassWithdraw(billId, remark);
            ServiceUtil.assertion(updateCount != 1, "提现审核失败，请刷新后重试");

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString());
        }
    }

    @Override
    @Transactional
    public boolean rejectWithdraw(Long billId, String remark) {
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString()), "该提现申请正在审核，请刷新后重试");

            int updateCount = smTransactionBillMapper.doRejectWithdraw(billId, remark);
            ServiceUtil.assertion(updateCount != 1, "提现审核失败，请刷新后重试");

            // 返还客户余额
            SmTransactionBill bill = smTransactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null || bill.getUserId() == null, "数据不存在");
            userService.addBalance(bill.getUserId(), bill.getMoney());

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString());
        }
    }

    /**
     * 提现打款
     * @param billId 提现单id
     */
    @Override
    @Transactional
    public boolean payWithdraw(Long billId) {
        ServiceUtil.assertion(billId == null, "参数错误：提现打款id不允许为空");
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.PAY_WITHDRAW, billId.toString()), "该提现申请正在打款，请刷新后重试");

            SmTransactionBill bill = smTransactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null, "单据不存在");

            // 更新为打款中
            int updateCount = smTransactionBillMapper.doPayWithdraw(billId);
            ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");

            // 发起打款
            if (TransactionBillPayType.WECHAT.getType().equals(bill.getPayType())) {
                InitiateBatchTransferResponse res = wxPayService.payWithdraw(billId);
                log.debug(String.valueOf(res));
            } else {
                throw new ServiceException("其他打款方式正在开发中");
            }

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.PAY_WITHDRAW, billId.toString());
        }
    }

    @Override
    @Transactional
    public void withdrawSuccess(Long billId) {
        // 修改状态
        int updateCount = smTransactionBillMapper.withdrawSuccess(billId);
        ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");
    }

    @Override
    @Transactional
    public void withdrawFailed(Long billId) {
        // 修改状态
        int updateCount = smTransactionBillMapper.withdrawFailed(billId);
        ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");
    }

    /**
     * 更新微信批次明细单号
     * @param billId 提现单号
     * @param transferIds 微信批次明细列表
     */
    @Override
    public boolean updateWxTransferDetailIds(Long billId, List<String> transferIds) {
        ServiceUtil.assertion(billId == null || CollectionUtils.isEmpty(transferIds), "参数错误：提现单号或微信批次明细列表为空");
        SmTransactionBill bill = new SmTransactionBill();
        bill.setBillId(billId);
        bill.setTransferIds(transferIds);
        return smTransactionBillMapper.updateSmTransactionBill(bill) == 1;
    }

    @Override
    public List<BillCountVo> selectLandlordCount(SmTransactionBillDto dto) {
        List<BillCountVo> list = this.selectCount(dto);

        // 若startDate与endDate不为空，则将其中间数据补全，不让其为空
        if (dto.getStartDate() != null && dto.getEndDate() != null && Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())) {
            List<BillCountVo> newList = new ArrayList<>();
            Date i = dto.getStartDate();
            while (i.compareTo(dto.getEndDate()) <= 0) {
                Date finalI = i;
                BillCountVo count = list.stream().filter(item -> DateUtils.getYYYY_MM_DD(item.getCreateDate()).equals(DateUtils.getYYYY_MM_DD(finalI))).findFirst().orElse(null);
                if (count == null) {
                    count = new BillCountVo();
                    count.setCreateDate(finalI);
                    count.setServiceCharge(BigDecimal.ZERO);
                    count.setWithdraw(BigDecimal.ZERO);
                    count.setRecharge(BigDecimal.ZERO);
                    count.setCreateYear(DateUtils.getYear(finalI));
                    count.setCreateMonth(DateUtils.getMonth(finalI));
                    count.setCreateDay(DateUtils.getDay(finalI));
                    count.setCreateYearMonth(DateUtils.getYYYY_MM(finalI));
                }
                newList.add(count);
                i = DateUtils.addDays(i,1);
            }

            // 列表排序
            newList.sort(Comparator.comparing(BillCountVo::getCreateDate));
            list = newList;
        }

        return list;
    }

    /**
     * 刷新支付结果
     * @param billIds
     */
    @Override
    @Transactional
    public void refreshPayResult(List<Long> billIds) {
        SmTransactionBillDto dto = new SmTransactionBillDto();
        dto.setBillIds(billIds);
        List<SmTransactionBillVo> billList = smTransactionBillMapper.selectSmTransactionBillList(dto);
        if (CollectionUtils.isEmpty(billList)) {
            return;
        }

        // 获取支付结果，并判断是否支付成功，若成功则更新数据
        Date now = new Date();
        for (SmTransactionBillVo bill : billList) {
            if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus()) || TransactionBillStatus.UNPAID.getStatus().equals(bill.getStatus())  ) {
                boolean payResult = getPayResult(bill.getBillNo());
                if (payResult) {
                    this.rechargeSuccess(bill.getBillNo(), now);
                }
            }
        }
    }

    @Override
    @Transactional
    public void paying(Long billId) {
        ServiceUtil.assertion(billId == null, "订单id不允许为空");
        int updateCount = smTransactionBillMapper.paying(billId);
        ServiceUtil.assertion(updateCount != 1, "订单状态发生变化，请刷新后重试");
    }

    /**
     * 手动设备充值
     *
     * @param billId
     */
    @Override
    @Transactional
    public boolean rechargeDevice(Long billId, boolean validate) {
        ServiceUtil.assertion(billId == null, "参数错误,billId不允许为空");

        SmTransactionBill bill = smTransactionBillMapper.selectSmTransactionBillByBillId(billId);

        if (validate) {
            ServiceUtil.assertion(bill == null || !TransactionBillType.RECHARGE.getType().equals(bill.getType()), "不存在的充值订单");
            ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "订单未支付");
            ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.SUCCESS.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已充值成功，不允许再次充值");
        }

        // 电表电量增加,四舍五入,保留1位小数
        deviceService.addElectricity(bill.getDeviceId(), computeElectricity(bill));

        // 修改设备充值状态：成功
        smTransactionBillMapper.updateDeviceRechargeStatus(bill.getBillId(), TransactionBillDeviceRechargeStatus.SUCCESS.getStatus());

        return true;
    }

    /**
     * 修改支付方式
     * @param billId 订单id
     * @param transactionBillPayType 支付方式
     * @param accountNo 账号（银行卡、openId）
     */
    @Override
    @Transactional
    public int updatePayType(Long billId, TransactionBillPayType transactionBillPayType, String accountNo) {
        ServiceUtil.assertion( billId == null, "订单id不允许为空");
        SmTransactionBill dbBill = smTransactionBillMapper.selectSmTransactionBillByBillId(billId);
        ServiceUtil.assertion(dbBill == null, "订单不存在");
        ServiceUtil.assertion(!TransactionBillStatus.UNPAID.getStatus().equals(dbBill.getStatus()), "只有未支付的订单可以更改支付方式");

        return smTransactionBillMapper.updatePayType(billId, transactionBillPayType.getType(), accountNo);
    }

    /**
     * 获取支付结果
     * @param billNo 订单编号
     * @return 支付结果
     */
    @Override
    @Transactional
    public boolean getPayResult(String billNo) {
        // 查询订单
        SmTransactionBill bill = smTransactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在");

        if (TransactionBillPayType.WECHAT.getType().equals(bill.getPayType())) {
            // 微信支付
            Transaction transaction = wxPayService.queryOrderByOutTradeNo(billNo);
            return wxPayService.isSuccess(transaction);
        }
        // ... TODO 其他支付方式

        return false;
    }

    /**
     * 充值成功
     * @param billNo 充值订单编号
     */
    @Override
    @Transactional
    public void rechargeSuccess(String billNo, Date payTime) {
        SmTransactionBill bill = smTransactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null || !TransactionBillType.RECHARGE.getType().equals(bill.getType()), "充值订单不存在");

        // 修改订单状态为成功
        int updateCount = smTransactionBillMapper.rechargeSuccess(bill.getBillId(), payTime);
        ServiceUtil.assertion(updateCount != 1, "修改订单状态失败，请刷新后重试");

        // 房东余额增加
        userService.addBalance(bill.getLandlordId(), bill.getArrivalAmount());

        // 设备充值
        rechargeDevice(bill.getBillId(), false);

        // 记录下充值后的余额
        SmUserVo user = userService.selectSmUserByUserId(bill.getLandlordId());
        updateAfterBalance(bill.getBillId(), user.getBalance());

    }

    // 计算充值电量
    private BigDecimal computeElectricity(SmTransactionBill bill) {
        if (bill == null) {
            return BigDecimal.ZERO;
        }
        return bill.getDeviceAmount().divide(bill.getUnitPrice(), 1, RoundingMode.HALF_UP);
    }

    /**
     * 记录下操作后的余额
     * @param billId 订单id
     * @param balance 余额
     */
    private void updateAfterBalance(Long billId, BigDecimal balance) {
        SmTransactionBill data = new SmTransactionBill();
        data.setBillId(billId);
        data.setAfterBalance(balance);
        smTransactionBillMapper.updateSmTransactionBill(data);
    }

    /**
     * 取消充值订单
     * @param billNo 订单编号
     * @param status 取消后的状态
     */
    @Override
    @Transactional
    public int cancelRecharge(String billNo, TransactionBillStatus status) {
        ServiceUtil.assertion(!TransactionBillStatus.isCancel(status.getStatus()), "参数错误：状态并非取消状态");

        SmTransactionBill bill = smTransactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在");
        // 用户自行取消，验证身份
        ServiceUtil.assertion(TransactionBillStatus.CANCELED.equals(status) && !UserUtil.hasFrontUser(bill.getUserId()), "这不是您的订单");

        ServiceUtil.assertion(!TransactionBillStatus.UNPAID.getStatus().equals(bill.getStatus()), "订单状态非未支付，不允许取消");
        ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "非充值订单，不允许取消");

        // 执行取消订单
        int updateCount = smTransactionBillMapper.cancelRecharge(billNo, status.getStatus());
        ServiceUtil.assertion(updateCount != 1, "订单取消失败，请刷新后重试");

        // 关闭支付订单
        if (TransactionBillPayType.WECHAT.getType().equals(bill.getPayType())) {
            wxPayService.closeOrder(billNo);
        }

        return updateCount;
    }

    @Override
    public SmTransactionBill selectSmTransactionBillByBillNo(String billNo) {
        return smTransactionBillMapper.selectSmTransactionBillByBillNo(billNo);
    }

    @Override
    public List<SmTransactionBillVo> selectUnpaidRecharge() {
        SmTransactionBillDto dto = new SmTransactionBillDto();
        dto.setStatus(TransactionBillStatus.UNPAID.getStatus());
        dto.setType(TransactionBillType.RECHARGE.getType());
        return this.selectSmTransactionBillList(dto);
    }

}
