package com.ruoyi.web.controller.app;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceDto;
import com.ruoyi.ss.meterReadingRecord.domain.SmMeterReadingRecordDto;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.meterReadingRecord.service.ISmMeterReadingRecordService;
import com.ruoyi.web.core.annotation.UserTypePermission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collections;

/**
 * 设备Controller
 * @author 辉
 * 2024/3/5
 */
@Api(tags = "设备")
@RestController
@RequestMapping("/app/device")
public class AppDeviceController extends BaseController {

    @Autowired
    private ISmDeviceService smDeviceService;

    @Autowired
    private ISmMeterReadingRecordService smMeterReadingRecordService;

    @Value("${debug}")
    private Boolean debug;

    /**
     * 修改设备
     */
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("房东修改设备")
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated({ValidGroup.FrontUpdate.class}) SmDevice smDevice) {
        SmDevice device = new SmDevice();
        device.setDeviceId(smDevice.getDeviceId());
        device.setGroupId(smDevice.getGroupId());
        device.setPrice(smDevice.getPrice());
        device.setOutageWay(smDevice.getOutageWay());
        device.setTenantBearServiceFee(smDevice.getTenantBearServiceFee());
        device.setRemark(smDevice.getRemark());
        device.setNoticeWay(smDevice.getNoticeWay());
        device.setEnableExpireNotice(smDevice.getEnableExpireNotice());
        device.setExpireDate(smDevice.getExpireDate());
        device.setExpireDay(smDevice.getExpireDay());
        return toAjax(smDeviceService.updateSmDevice(device));
    }

    /**
     * 查询设备列表
     */
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("查询房东设备列表")
    @GetMapping("/list")
    public TableDataInfo list(SmDeviceDto smDevice) {
        startPage();
        smDevice.setUserId(getUserId());
        return getDataTable(smDeviceService.selectSmDeviceList(smDevice));
    }

    /**
     * 获取设备详细信息
     */
    @ApiOperation("获取设备详细信息")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId) {
        return success(smDeviceService.selectSmDeviceByDeviceId(deviceId));
    }

    /**
     * 解除设备绑定（删除设备）
     * @param deviceId 设备id
     * @return
     */
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("（弃用）房东解除设备绑定（删除设备）")
    @DeleteMapping("/{deviceId}")
    public AjaxResult delete(@PathVariable Long deviceId) {
        ServiceUtil.assertion(true, "本接口已弃用，请使用【DELETE】 /app/device/landlord/unbind/{deviceId}接口");
        return success(smDeviceService.logicDel(Collections.singletonList(deviceId)));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("房东新增设备")
    @PostMapping
    public AjaxResult add(@RequestBody @Validated({ValidGroup.FrontCreate.class}) SmDevice smDevice) {
        SmDevice insertDevice = new SmDevice();
        insertDevice.setMac(smDevice.getMac());
        insertDevice.setUserId(getUserId());
        insertDevice.setDeviceName(smDevice.getMac());
        return success(smDeviceService.insertSmDevice(insertDevice));
    }

    @UserTypePermission({UserType.TENANT})
    @ApiOperation("获取租户设备信息")
    @GetMapping("/tenant")
    public TableDataInfo tenant(SmDeviceDto dto) {
        startPage();
        dto.setTenantId(getUserId());
        return getDataTable(smDeviceService.selectSmDeviceList(dto));
    }

    @UserTypePermission({UserType.TENANT})
    @ApiOperation("租户绑定设备")
    @PutMapping("/tenant/bind/{deviceId}")
    public AjaxResult tenantBind(@PathVariable @ApiParam("设备id") Long deviceId) {
        ServiceUtil.assertion(true, "本接口已弃用，请使用【PUT】 /app/device/bind/{deviceId}接口");
        return success(smDeviceService.tenantBind(getUserId(), deviceId));
    }

    @UserTypePermission({UserType.TENANT})
    @ApiOperation("租户解绑设备")
    @PutMapping("/tenant/unbind/{deviceId}")
    public AjaxResult tenantUnbind(@PathVariable @ApiParam("设备id") Long deviceId) {
        return success(smDeviceService.tenantUnbind(getUserId(), deviceId));
    }

    @UserTypePermission({UserType.TENANT})
    @ApiOperation("租户切换默认设备")
    @PutMapping("/tenant/setDefault/{deviceId}")
    public AjaxResult setDefault(@PathVariable @ApiParam("设备id") Long deviceId) {
        return success(smDeviceService.setDefaultDevice(getUserId(), deviceId));
    }

    @ApiOperation("设备通断电")
    @PutMapping("/{deviceId}/changePower")
    public AjaxResult close(@PathVariable @ApiParam("设备id") Long deviceId, String status) {
        DevicePowerStatus powerStatus = DevicePowerStatus.parse(status);
        return success(smDeviceService.changePowerStatus(deviceId, powerStatus ));
    }

    @ApiOperation("绑定设备")
    @PutMapping("/bind/{mac}")
    public AjaxResult bind(@PathVariable @ApiParam("设备mac") String mac) {
        return success(smDeviceService.bind(getUserId(), mac));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("房东解除设备绑定")
    @DeleteMapping("/landlord/unbind/{deviceId}")
    public AjaxResult landlordUnbind(@PathVariable @ApiParam("设备id") Long deviceId) {
        return success(smDeviceService.landlordUnbind(getUserId(), deviceId));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("设备电量归零")
    @PutMapping("{deviceId}/reset")
    public AjaxResult reset(@PathVariable @ApiParam("设备id") Long deviceId) {
        return success(smDeviceService.reset(deviceId));
    }

    @ApiOperation("获取设备用电量分析")
    @GetMapping("/electric/count")
    public AjaxResult count(@Validated({ValidGroup.Query.class}) SmMeterReadingRecordDto dto)
    {
        return success(smMeterReadingRecordService.selectCount(dto));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("设备充值电量")
    @PostMapping("/addElectricity/{deviceId}")
    public AjaxResult mockRecharge(@PathVariable @ApiParam("设备id") Long deviceId, @ApiParam("电量（度）") BigDecimal amount)
    {
        SmDeviceVo device = smDeviceService.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || !getUserId().equals(device.getUserId()), "设备不存在或您无权充值");

        smDeviceService.addElectricity(deviceId, amount);
        return success(true);
    }

    @ApiOperation("刷新设备信息")
    @GetMapping("/{deviceId}/refreshIot")
    public AjaxResult syncIot(@PathVariable @ApiParam("设备id") Long deviceId) {
        smDeviceService.pullDeviceInfo(Collections.singletonList(deviceId));
        return success();
    }

    @ApiOperation("判断设备是否已经被绑定")
    @GetMapping("/{mac}/isBind")
    public AjaxResult syncIot(@PathVariable @ApiParam("设备mac") String mac) {
        return success(smDeviceService.isBind(mac));
    }
}
