package com.ruoyi.web.controller.app;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroup;
import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroupVo;
import com.ruoyi.ss.deviceGroup.service.ISmDeviceGroupService;
import com.ruoyi.web.core.annotation.UserTypePermission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 设备分组Controller
 * @author 辉
 * 2024/3/5
 */
@Api(tags = "设备分组")
@RestController
@RequestMapping("/app/group")
public class AppDeviceGroupController extends BaseController {

    @Autowired
    private ISmDeviceGroupService smDeviceGroupService;

    /**
     * 查询设备分组列表
     */
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("查询设备分组列表")
    @GetMapping("/list")
    public TableDataInfo list(SmDeviceGroup smDeviceGroup) {
        startPage();
        smDeviceGroup.setUserId(getUserId());
        List<SmDeviceGroupVo> list = smDeviceGroupService.selectSmDeviceGroupList(smDeviceGroup);
        return getDataTable(list);
    }

    /**
     * 修改设备分组信息
     * @param smDeviceGroup
     * @return
     */
    @DataScope
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("修改设备分组信息")
    @PutMapping
    public AjaxResult edit(@RequestBody SmDeviceGroup smDeviceGroup) {
        smDeviceGroup.setCreateTime(null);
        smDeviceGroup.setCreateBy(null);
        smDeviceGroup.setUserId(null);
        return AjaxResult.success(smDeviceGroupService.updateSmDeviceGroup(smDeviceGroup));
    }

    /**
     * 删除设备分组信息
     * @param groupId
     * @return
     */
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("删除设备分组信息")
    @DeleteMapping("/{groupId}")
    public AjaxResult delete(@PathVariable Long groupId) {
        return AjaxResult.success(smDeviceGroupService.deleteSmDeviceGroupByGroupId(groupId));
    }

    /**
     * 新增设备分组
     * @param smDeviceGroup
     * @return
     */
    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("新增设备分组")
    @PostMapping
    public AjaxResult add(@RequestBody SmDeviceGroup smDeviceGroup) {
        smDeviceGroup.setUserId(getUserId());
        return AjaxResult.success(smDeviceGroupService.insertSmDeviceGroup(smDeviceGroup));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("调整分组位置")
    @PutMapping("/changeSort")
    public AjaxResult changeSort(@RequestBody List<SmDeviceGroup> list) {
        return AjaxResult.success(smDeviceGroupService.changeSort(list));
    }

    @ApiOperation("获取当前用户分组列表及其数量")
    @GetMapping("/listCount")
    public AjaxResult listCount(SmDeviceGroup smDeviceGroup) {
        smDeviceGroup.setUserId(getUserId());
        return AjaxResult.success(smDeviceGroupService.listCount(smDeviceGroup));
    }
}
