package com.ruoyi.web.controller.app;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBill;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillBO;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillDto;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillGroupBy;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.domain.SmTransactionBillVo;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.transactionBill.service.ISmTransactionBillService;
import com.ruoyi.web.core.annotation.UserTypePermission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 充值记录Controller
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Api(tags = "充值/提现记录")
@RestController
@RequestMapping("/app/bill")
public class AppTransactionBillController extends BaseController
{
    @Autowired
    private ISmTransactionBillService smTransactionBillService;

    /**
     * 查询充值记录列表
     */
    @ApiOperation("查询本人充值记录列表")
    @GetMapping("/rechargeList")
    public TableDataInfo rechargeList(SmTransactionBillDto smTransactionBill)
    {
        smTransactionBill.setUserId(getUserId());
        smTransactionBill.setType(TransactionBillType.RECHARGE.getType());
        startPage();
        List<SmTransactionBillVo> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        return getDataTable(list);
    }

    /**
     * 查询提现记录列表
     */
    @ApiOperation("查询本人提现记录列表")
    @GetMapping("/withdrawList")
    public TableDataInfo withdrawList(SmTransactionBillDto smTransactionBill)
    {
        startPage();
        smTransactionBill.setUserId(getUserId());
        smTransactionBill.setType(TransactionBillType.WITHDRAW.getType());
        List<SmTransactionBillVo> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        return getDataTable(list);
    }

    /**
     * 查询收支列表
     */
    @ApiOperation("查询本人收支列表")
    @GetMapping("/list")
    public TableDataInfo list(SmTransactionBillDto smTransactionBill)
    {
        startPage();
        smTransactionBill.setUserId(getUserId());
        List<SmTransactionBillVo> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        return getDataTable(list);
    }

    @ApiOperation("获取记录详情")
    @GetMapping("/{billId}")
    public AjaxResult detail(@PathVariable Long billId) {
        return AjaxResult.success(smTransactionBillService.selectSmTransactionBillByBillId(billId));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("获取房东统计信息")
    @GetMapping("/landlordCount")
    public AjaxResult landlordCount(@Validated({ValidGroup.Query.class}) SmTransactionBillDto dto) {
        Date now = new Date();
        if (dto.getEndDate() == null) {
            dto.setEndDate(now);
        }
        // 如果按日查询，查询的月份是当前月份，并且当前日期小于7日，则查询最近7天的数据，否则查询当月第一天开始的数据
        if (Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())
                && Objects.equals(dto.getYear(), DateUtils.getYear(now))
                && Objects.equals(dto.getMonth(), DateUtils.getMonth(now))
                && DateUtils.getDay(now) < 7) {
            // 七天前
            dto.setStartDate(DateUtils.addDays(dto.getEndDate(), -6));
        } else {
            // 每月第一天
            Date firstDay = new Date(dto.getEndDate().getTime());
            firstDay.setDate(1);
            dto.setStartDate(firstDay);
        }
        dto.setYear(null);
        dto.setMonth(null);
        dto.setLandlordId(getUserId());
        dto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        AjaxResult ajax = AjaxResult.success(smTransactionBillService.selectLandlordCount(dto));

        // 总收入
        if (Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())) {
            dto.setGroupBy(TransactionBillGroupBy.create_month.name());
        } else if (Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_month.name())) {
            dto.setGroupBy(TransactionBillGroupBy.create_year.name());
        } else {
            dto.setGroupBy(null);
        }
        dto.setStartDate(null);
        dto.setMonth(DateUtils.getMonth(dto.getEndDate()));
        dto.setYear(DateUtils.getYear(dto.getEndDate()));
        dto.setEndDate(null);
        List<BillCountVo> totalList = smTransactionBillService.selectCount(dto);
        if (CollectionUtils.isEmpty(totalList)) {
            ajax.put("totalRecharge", 0);
        } else {
            ajax.put("totalRecharge", totalList.get(0).getRecharge());
        }

        return ajax;
    }

    @ApiOperation("创建充值订单")
    @PostMapping("/recharge")
    public AjaxResult addRecharge(@RequestBody @Validated({ValidGroup.Recharge.class}) SmTransactionBill body) {
        body.setUserId(getUserId());
        return AjaxResult.success("操作成功", smTransactionBillService.addRecharge(body));
    }

    @ApiOperation("取消充值订单")
    @PutMapping("/recharge/cancel/{billNo}")
    public AjaxResult cancelRecharge(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(smTransactionBillService.cancelRecharge(billNo, TransactionBillStatus.CANCELED));
    }

    @UserTypePermission({UserType.LANDLORD})
    @ApiOperation("提现申请")
    @PostMapping("/withdraw")
    public AjaxResult withdraw(@RequestBody @Validated({ValidGroup.Withdraw.class}) SmTransactionBillBO data) {
        data.setUserId(getUserId());
        return AjaxResult.success("操作成功", smTransactionBillService.addWithdraw(data));
    }

}
