package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroupVo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroup;
import com.ruoyi.ss.deviceGroup.service.ISmDeviceGroupService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 设备分组Controller
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@RestController
@RequestMapping("/system/group")
public class SmDeviceGroupController extends BaseController
{
    @Autowired
    private ISmDeviceGroupService smDeviceGroupService;

    /**
     * 查询设备分组列表
     */
    @PreAuthorize("@ss.hasPermi('system:group:list')")
    @GetMapping("/list")
    public TableDataInfo list(SmDeviceGroup smDeviceGroup)
    {
        startPage();
        List<SmDeviceGroupVo> list = smDeviceGroupService.selectSmDeviceGroupList(smDeviceGroup);
        return getDataTable(list);
    }

    /**
     * 导出设备分组列表
     */
    @PreAuthorize("@ss.hasPermi('system:group:export')")
    @Log(title = "设备分组", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SmDeviceGroup smDeviceGroup)
    {
        List<SmDeviceGroupVo> list = smDeviceGroupService.selectSmDeviceGroupList(smDeviceGroup);
        ExcelUtil<SmDeviceGroupVo> util = new ExcelUtil<SmDeviceGroupVo>(SmDeviceGroupVo.class);
        util.exportExcel(response, list, "设备分组数据");
    }

    /**
     * 获取设备分组详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:group:query')")
    @GetMapping(value = "/{groupId}")
    public AjaxResult getInfo(@PathVariable("groupId") Long groupId)
    {
        return success(smDeviceGroupService.selectSmDeviceGroupByGroupId(groupId));
    }

    /**
     * 新增设备分组
     */
    @PreAuthorize("@ss.hasPermi('system:group:add')")
    @Log(title = "设备分组", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SmDeviceGroup smDeviceGroup)
    {
        return toAjax(smDeviceGroupService.insertSmDeviceGroup(smDeviceGroup));
    }

    /**
     * 修改设备分组
     */
    @PreAuthorize("@ss.hasPermi('system:group:edit')")
    @Log(title = "设备分组", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SmDeviceGroup smDeviceGroup)
    {
        return toAjax(smDeviceGroupService.updateSmDeviceGroup(smDeviceGroup));
    }

    /**
     * 删除设备分组
     */
    @PreAuthorize("@ss.hasPermi('system:group:remove')")
    @Log(title = "设备分组", businessType = BusinessType.DELETE)
	@DeleteMapping("/{groupIds}")
    public AjaxResult remove(@PathVariable Long[] groupIds)
    {
        return toAjax(smDeviceGroupService.deleteSmDeviceGroupByGroupIds(groupIds));
    }
}
