package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.ss.user.domain.SmUserDto;
import com.ruoyi.ss.user.domain.SmUserVo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.core.domain.entity.SmUser;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 普通用户信息Controller
 *
 * @author qzz
 * @date 2024-01-24
 */
@RestController
@RequestMapping("/system/smUser")
public class SmUserController extends BaseController
{
    @Autowired
    private ISmUserService smUserService;

    /**
     * 查询普通用户信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:list')")
    @GetMapping("/list")
    public TableDataInfo list(SmUserDto smUser)
    {
        startPage();
        List<SmUserVo> list = smUserService.selectSmUserList(smUser);
        return getDataTable(list);
    }

    /**
     * 导出普通用户信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:export')")
    @Log(title = "普通用户信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SmUserDto smUser)
    {
        List<SmUserVo> list = smUserService.selectSmUserList(smUser);
        ExcelUtil<SmUserVo> util = new ExcelUtil<SmUserVo>(SmUserVo.class);
        util.exportExcel(response, list, "普通用户信息数据");
    }

    /**
     * 获取普通用户信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:query')")
    @GetMapping(value = "/{userId}")
    public AjaxResult getInfo(@PathVariable("userId") Long userId)
    {
        return success(smUserService.selectSmUserByUserId(userId));
    }

    /**
     * 新增普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:add')")
    @Log(title = "普通用户信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SmUser smUser)
    {
        return toAjax(smUserService.insertSmUser(smUser));
    }

    /**
     * 修改普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:edit')")
    @Log(title = "普通用户信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SmUser smUser)
    {
        return toAjax(smUserService.updateSmUser(smUser));
    }

    /**
     * 删除普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:remove')")
    @Log(title = "普通用户信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{userIds}")
    public AjaxResult remove(@PathVariable Long[] userIds)
    {
        return toAjax(smUserService.deleteSmUserByUserIds(userIds));
    }

}
