package com.ruoyi.web.core.aspectj;

import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.web.core.annotation.UserTypePermission;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户类型权限切面
 * @author 辉
 * 2024/3/8
 */
@Aspect
@Component
public class UserTypePermissionAspect {

    @Autowired
    private ISmUserService smUserService;

    @Value("${debug}")
    private Boolean debug;

    // 判断当前用户是否是对应的类型
    @Before("@annotation(userTypePermission)")
    public void doBefore(JoinPoint point, UserTypePermission userTypePermission) {
        if (debug) {
            return;
        }
        SmUserVo user = smUserService.selectSimpleById(SecurityUtils.getUserId());
        List<String> userTypes = Arrays.stream(userTypePermission.value()).map(UserType::getType).collect(Collectors.toList());
        ServiceUtil.assertion(!userTypes.contains(user.getUserType()), "您无权访问该方法");
    }

}
